@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1999-2008   The R Development Core Team.
+ *  Copyright (C) 1999-2010   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +22,15 @@
 #define R_INTERNALS_H_
 
 #ifdef __cplusplus
+# include <cstdio>
+# ifdef __SUNPRO_CC
+using std::FILE;
+# endif
+# include <climits>
 extern "C" {
+#else
+# include <stdio.h>
+# include <limits.h> /* for INT_MAX */
 #endif
 
 #include <R_ext/Arith.h>
@@ -33,17 +41,6 @@ extern "C" {
 #include <R_ext/PrtUtil.h>
 #include <R_ext/Utils.h>
 
-#include <stdio.h> /* for FILE */
-/*
-#include <errno.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <limits.h>
-#include <float.h>
-#include <ctype.h>
-*/
-
 #include <R_ext/libextern.h>
 
 typedef unsigned char Rbyte;
@@ -104,6 +101,10 @@ typedef unsigned int SEXPTYPE;
 #define RAWSXP      24    /* raw bytes */
 #define S4SXP       25    /* S4, non-vector */
 
+/* used for detecting PROTECT issues in memory.c */
+#define NEWSXP      30    /* fresh node creaed in new page */
+#define FREESXP     31    /* node released by GC */
+
 #define FUNSXP      99    /* Closure or Builtin or Special */
 
 
@@ -129,11 +130,14 @@ typedef enum {
     ANYSXP	= 18,	/* make "any" args work */
     VECSXP	= 19,	/* generic vectors */
     EXPRSXP	= 20,	/* expressions vectors */
-    BCODESXP    = 21,   /* byte code */
-    EXTPTRSXP   = 22,   /* external pointer */
-    WEAKREFSXP  = 23,   /* weak reference */
-    RAWSXP      = 24,   /* raw bytes */
-    S4SXP         = 25,   /* S4 non-vector */
+    BCODESXP	= 21,	/* byte code */
+    EXTPTRSXP	= 22,	/* external pointer */
+    WEAKREFSXP	= 23,	/* weak reference */
+    RAWSXP	= 24,	/* raw bytes */
+    S4SXP	= 25,	/* S4 non-vector */
+
+    NEWSXP      = 30,   /* fresh node creaed in new page */
+    FREESXP     = 31,   /* node released by GC */
 
     FUNSXP	= 99	/* Closure or Builtin */
 } SEXPTYPE;
@@ -245,12 +249,12 @@ typedef union { VECTOR_SEXPREC s; double align; } SEXPREC_ALIGN;
 #define MARK(x)		((x)->sxpinfo.mark)
 #define TYPEOF(x)	((x)->sxpinfo.type)
 #define NAMED(x)	((x)->sxpinfo.named)
-#define TRACE(x)	((x)->sxpinfo.trace)
+#define RTRACE(x)	((x)->sxpinfo.trace)
 #define LEVELS(x)	((x)->sxpinfo.gp)
 #define SET_OBJECT(x,v)	(((x)->sxpinfo.obj)=(v))
 #define SET_TYPEOF(x,v)	(((x)->sxpinfo.type)=(v))
 #define SET_NAMED(x,v)	(((x)->sxpinfo.named)=(v))
-#define SET_TRACE(x,v)	(((x)->sxpinfo.trace)=(v))
+#define SET_RTRACE(x,v)	(((x)->sxpinfo.trace)=(v))
 #define SETLEVELS(x,v)	(((x)->sxpinfo.gp)=(v))
 
 /* S4 object bit, set by R_do_new_object for all new() calls */
@@ -306,8 +310,10 @@ typedef union { VECTOR_SEXPREC s; double align; } SEXPREC_ALIGN;
 #define FORMALS(x)	((x)->u.closxp.formals)
 #define BODY(x)		((x)->u.closxp.body)
 #define CLOENV(x)	((x)->u.closxp.env)
-#define DEBUG(x)	((x)->sxpinfo.debug)
-#define SET_DEBUG(x,v)	(((x)->sxpinfo.debug)=(v))
+#define RDEBUG(x)	((x)->sxpinfo.debug)
+#define SET_RDEBUG(x,v)	(((x)->sxpinfo.debug)=(v))
+#define RSTEP(x)	((x)->sxpinfo.spare)
+#define SET_RSTEP(x,v)	(((x)->sxpinfo.spare)=(v))
 
 /* Symbol Access Macros */
 #define PRINTNAME(x)	((x)->u.symsxp.pname)
@@ -417,10 +423,12 @@ SEXP SETCAD4R(SEXP e, SEXP y);
 SEXP (FORMALS)(SEXP x);
 SEXP (BODY)(SEXP x);
 SEXP (CLOENV)(SEXP x);
-int  (DEBUG)(SEXP x);
-int  (TRACE)(SEXP x);
-void (SET_DEBUG)(SEXP x, int v);
-void (SET_TRACE)(SEXP x, int v);
+int  (RDEBUG)(SEXP x);
+int  (RSTEP)(SEXP x);
+int  (RTRACE)(SEXP x);
+void (SET_RDEBUG)(SEXP x, int v);
+void (SET_RSTEP)(SEXP x, int v);
+void (SET_RTRACE)(SEXP x, int v);
 void SET_FORMALS(SEXP x, SEXP v);
 void SET_BODY(SEXP x, SEXP v);
 void SET_CLOENV(SEXP x, SEXP v);
@@ -470,20 +478,16 @@ void (SET_HASHVALUE)(SEXP x, int v);
 #define EXTPTR_PROT(x)	CDR(x)
 #define EXTPTR_TAG(x)	TAG(x)
 
-#ifdef BYTECODE
 /* Bytecode access macros */
 #define BCODE_CODE(x)	CAR(x)
 #define BCODE_CONSTS(x) CDR(x)
 #define BCODE_EXPR(x)	TAG(x)
 #define isByteCode(x)	(TYPEOF(x)==BCODESXP)
-#else
-#define isByteCode(x)	FALSE
-#endif
 
 /* Pointer Protection and Unprotection */
-#define PROTECT(s)	protect(s)
-#define UNPROTECT(n)	unprotect(n)
-#define UNPROTECT_PTR(s)	unprotect_ptr(s)
+#define PROTECT(s)	Rf_protect(s)
+#define UNPROTECT(n)	Rf_unprotect(n)
+#define UNPROTECT_PTR(s)	Rf_unprotect_ptr(s)
 
 /* We sometimes need to coerce a protected value and place the new
    coerced value under protection.  For these cases PROTECT_WITH_INDEX
@@ -499,8 +503,10 @@ LibExtern SEXP	R_GlobalEnv;	    /* The "global" environment */
 LibExtern SEXP  R_EmptyEnv;	    /* An empty environment at the root of the
 				    	environment tree */
 LibExtern SEXP  R_BaseEnv;	    /* The base environment; formerly R_NilValue */
-LibExtern SEXP	R_BaseNamespace;    /* The (fake) name space for base */
-LibExtern SEXP	R_NamespaceRegistry;/* Registry for registered name spaces */
+LibExtern SEXP	R_BaseNamespace;    /* The (fake) namespace for base */
+LibExtern SEXP	R_NamespaceRegistry;/* Registry for registered namespaces */
+
+LibExtern SEXP	R_Srcref;           /* Current srcref, for debuggers */
 
 /* Special Values */
 LibExtern SEXP	R_NilValue;	    /* The nil object */
@@ -518,23 +524,38 @@ LibExtern SEXP	R_Bracket2Symbol;   /* "[[" */
 LibExtern SEXP	R_BracketSymbol;    /* "[" */
 LibExtern SEXP	R_BraceSymbol;      /* "{" */
 LibExtern SEXP	R_ClassSymbol;	    /* "class" */
+LibExtern SEXP	R_DeviceSymbol;     /* ".Device" */
 LibExtern SEXP	R_DimNamesSymbol;   /* "dimnames" */
 LibExtern SEXP	R_DimSymbol;	    /* "dim" */
 LibExtern SEXP	R_DollarSymbol;	    /* "$" */
 LibExtern SEXP	R_DotsSymbol;	    /* "..." */
 LibExtern SEXP	R_DropSymbol;	    /* "drop" */
+LibExtern SEXP	R_LastvalueSymbol;  /* ".Last.value" */
 LibExtern SEXP	R_LevelsSymbol;	    /* "levels" */
 LibExtern SEXP	R_ModeSymbol;	    /* "mode" */
+LibExtern SEXP	R_NameSymbol;	    /* "name" */
 LibExtern SEXP	R_NamesSymbol;	    /* "names" */
+LibExtern SEXP	R_NaRmSymbol;	    /* "na.rm" */
+LibExtern SEXP  R_PackageSymbol;    /* "package" */
+LibExtern SEXP  R_QuoteSymbol;	    /* "quote" */
 LibExtern SEXP	R_RowNamesSymbol;   /* "row.names" */
 LibExtern SEXP	R_SeedsSymbol;	    /* ".Random.seed" */
+LibExtern SEXP	R_SourceSymbol;     /* "source" */
 LibExtern SEXP	R_TspSymbol;	    /* "tsp" */
 
+LibExtern SEXP  R_dot_defined;      /* ".defined" */
+LibExtern SEXP  R_dot_Method;       /* ".Method" */
+LibExtern SEXP  R_dot_target;       /* ".target" */
+
 /* Missing Values - others from Arith.h */
 #define NA_STRING	R_NaString
 LibExtern SEXP	R_NaString;	    /* NA_STRING as a CHARSXP */
 LibExtern SEXP	R_BlankString;	    /* "" as a CHARSXP */
 
+/* srcref related functions */
+SEXP R_GetCurrentSrcref(int);
+SEXP R_GetSrcFilename(SEXP);
+
 /*--- FUNCTIONS ------------------------------------------------------ */
 
 /* Type Coercions of all kinds */
@@ -543,6 +564,7 @@ SEXP Rf_asChar(SEXP);
 SEXP Rf_coerceVector(SEXP, SEXPTYPE);
 SEXP Rf_PairToVectorList(SEXP x);
 SEXP Rf_VectorToPairList(SEXP x);
+SEXP Rf_asCharacterFactor(SEXP x);
 int Rf_asLogical(SEXP x);
 int Rf_asInteger(SEXP x);
 double Rf_asReal(SEXP x);
@@ -560,18 +582,17 @@ SEXP Rf_allocList(int);
 SEXP Rf_allocS4Object(void);
 SEXP Rf_allocSExp(SEXPTYPE);
 SEXP Rf_allocVector(SEXPTYPE, R_len_t);
+int  Rf_any_duplicated(SEXP x, Rboolean from_last);
+int  Rf_any_duplicated3(SEXP x, SEXP incomp, Rboolean from_last);
 SEXP Rf_applyClosure(SEXP, SEXP, SEXP, SEXP, SEXP);
 SEXP Rf_arraySubscript(int, SEXP, SEXP, SEXP (*)(SEXP,SEXP),
                        SEXP (*)(SEXP, int), SEXP);
-Rcomplex Rf_asComplex(SEXP);
-int Rf_asInteger(SEXP);
-int Rf_asLogical(SEXP);
-double Rf_asReal(SEXP);
 SEXP Rf_classgets(SEXP, SEXP);
 SEXP Rf_cons(SEXP, SEXP);
 void Rf_copyMatrix(SEXP, SEXP, Rboolean);
 void Rf_copyMostAttrib(SEXP, SEXP);
 void Rf_copyVector(SEXP, SEXP);
+int Rf_countContexts(int, int);
 SEXP Rf_CreateTag(SEXP);
 void Rf_defineVar(SEXP, SEXP, SEXP);
 SEXP Rf_dimgets(SEXP, SEXP);
@@ -588,26 +609,29 @@ SEXP Rf_getAttrib(SEXP, SEXP);
 SEXP Rf_GetArrayDimnames(SEXP);
 SEXP Rf_GetColNames(SEXP);
 void Rf_GetMatrixDimnames(SEXP, SEXP*, SEXP*, const char**, const char**);
-SEXP Rf_GetOption(SEXP, SEXP);
-int Rf_GetOptionDigits(SEXP);
-int Rf_GetOptionWidth(SEXP);
+SEXP Rf_GetOption(SEXP, SEXP); /* pre-2.13.0 compatibility */
+SEXP Rf_GetOption1(SEXP);
+int Rf_GetOptionDigits(void);
+int Rf_GetOptionWidth(void);
 SEXP Rf_GetRowNames(SEXP);
 void Rf_gsetVar(SEXP, SEXP, SEXP);
 SEXP Rf_install(const char *);
 Rboolean Rf_isFree(SEXP);
 Rboolean Rf_isOrdered(SEXP);
 Rboolean Rf_isUnordered(SEXP);
-Rboolean Rf_isUnsorted(SEXP);
+Rboolean Rf_isUnsorted(SEXP, Rboolean);
 SEXP Rf_lengthgets(SEXP, R_len_t);
 SEXP R_lsInternal(SEXP, Rboolean);
 SEXP Rf_match(SEXP, SEXP, int);
+SEXP Rf_matchE(SEXP, SEXP, int, SEXP);
 SEXP Rf_namesgets(SEXP, SEXP);
 SEXP Rf_mkChar(const char *);
-SEXP Rf_mkCharEnc(const char *, int);
+SEXP Rf_mkCharLen(const char *, int);
 Rboolean Rf_NonNullStringMatch(SEXP, SEXP);
 int Rf_ncols(SEXP);
 int Rf_nrows(SEXP);
 SEXP Rf_nthcdr(SEXP, int);
+
 Rboolean Rf_pmatch(SEXP, SEXP, Rboolean);
 Rboolean Rf_psmatch(const char *, const char *, Rboolean);
 void Rf_PrintValue(SEXP);
@@ -619,6 +643,8 @@ SEXPTYPE Rf_str2type(const char *);
 Rboolean Rf_StringBlank(SEXP);
 SEXP Rf_substitute(SEXP,SEXP);
 const char * Rf_translateChar(SEXP);
+const char * Rf_translateChar0(SEXP);
+const char * Rf_translateCharUTF8(SEXP);
 const char * Rf_type2char(SEXPTYPE);
 SEXP Rf_type2str(SEXPTYPE);
 void Rf_unprotect(int);
@@ -627,9 +653,27 @@ void Rf_unprotect_ptr(SEXP);
 void R_ProtectWithIndex(SEXP, PROTECT_INDEX *);
 void R_Reprotect(SEXP, PROTECT_INDEX);
 SEXP R_tryEval(SEXP, SEXP, int *);
+SEXP R_tryEvalSilent(SEXP, SEXP, int *);
+const char *R_curErrorBuf();
 
 Rboolean Rf_isS4(SEXP);
-SEXP Rf_asS4(SEXP, Rboolean);
+SEXP Rf_asS4(SEXP, Rboolean, int);
+SEXP Rf_S3Class(SEXP);
+int Rf_isBasicClass(const char *);
+
+typedef enum {
+    CE_NATIVE = 0,
+    CE_UTF8   = 1,
+    CE_LATIN1 = 2,
+    CE_BYTES  = 3,
+    CE_SYMBOL = 5,
+    CE_ANY    =99
+} cetype_t;
+
+cetype_t Rf_getCharCE(SEXP);
+SEXP Rf_mkCharCE(const char *, cetype_t);
+SEXP Rf_mkCharLenCE(const char *, int, cetype_t);
+const char *Rf_reEnc(const char *x, cetype_t ce_in, cetype_t ce_out, int subst);
 
 				/* return(.) NOT reached : for -Wall */
 #define error_return(msg)	{ Rf_error(msg);	   return R_NilValue; }
@@ -664,7 +708,6 @@ SEXP R_WeakRefKey(SEXP w);
 SEXP R_WeakRefValue(SEXP w);
 void R_RunWeakRefFinalizer(SEXP w);
 
-#ifdef BYTECODE
 SEXP R_PromiseExpr(SEXP);
 SEXP R_ClosureExpr(SEXP);
 void R_initialize_bcode(void);
@@ -672,10 +715,6 @@ SEXP R_bcEncode(SEXP);
 SEXP R_bcDecode(SEXP);
 #define PREXPR(e) R_PromiseExpr(e)
 #define BODY_EXPR(e) R_ClosureExpr(e)
-#else
-#define PREXPR(e) PRCODE(e)
-#define BODY_EXPR(e) BODY(e)
-#endif
 
 /* Protected evaluation */
 Rboolean R_ToplevelExec(void (*fun)(void *), void *data);
@@ -783,11 +822,15 @@ SEXP R_Unserialize(R_inpstream_t ips);
 /* slot management (in attrib.c) */
 SEXP R_do_slot(SEXP obj, SEXP name);
 SEXP R_do_slot_assign(SEXP obj, SEXP name, SEXP value);
+int R_has_slot(SEXP obj, SEXP name);
 
-/* class definition, new objects */
+/* class definition, new objects (objects.c) */
 SEXP R_do_MAKE_CLASS(const char *what);
 SEXP R_getClassDef  (const char *what);
 SEXP R_do_new_object(SEXP class_def);
+/* supporting  a C-level version of  is(., .) : */
+int R_check_class_and_super(SEXP x, const char **valid, SEXP rho);
+int R_check_class_etc      (SEXP x, const char **valid);
 
 /* preserve objects across GCs */
 void R_PreserveObject(SEXP);
@@ -803,9 +846,15 @@ FILE *R_popen(const char *, const char *);
 #endif
 int R_system(const char *);
 
-/* now a macro */
-#define allocString(n)		Rf_allocVector(CHARSXP, n)
-#define Rf_allocString(n)      	Rf_allocVector(CHARSXP, n)
+/* R_compute_identical:  C version of identical() function
+   The third arg to R_compute_identical() consists of bitmapped flags for non-default options:
+   currently all default to TRUE, so the flag is set for FALSE values:
+   1 = !NUM_EQ
+   2 = !SINGLE_NA
+   4 = !ATTR_AS_SET
+   8 = !IGNORE_BYTECODE
+*/
+Rboolean R_compute_identical(SEXP, SEXP, int);
 
 #ifndef R_NO_REMAP
 #define acopy_string		Rf_acopy_string
@@ -816,9 +865,12 @@ int R_system(const char *);
 #define allocS4Object		Rf_allocS4Object
 #define allocSExp		Rf_allocSExp
 #define allocVector		Rf_allocVector
+#define any_duplicated		Rf_any_duplicated
+#define any_duplicated3		Rf_any_duplicated3
 #define applyClosure		Rf_applyClosure
 #define arraySubscript		Rf_arraySubscript
 #define asChar			Rf_asChar
+#define asCharacterFactor	Rf_asCharacterFactor
 #define asComplex		Rf_asComplex
 #define asInteger		Rf_asInteger
 #define asLogical		Rf_asLogical
@@ -831,6 +883,7 @@ int R_system(const char *);
 #define copyMatrix		Rf_copyMatrix
 #define copyMostAttrib		Rf_copyMostAttrib
 #define copyVector		Rf_copyVector
+#define countContexts		Rf_countContexts
 #define CreateTag		Rf_CreateTag
 #define defineVar		Rf_defineVar
 #define dimgets			Rf_dimgets
@@ -847,16 +900,19 @@ int R_system(const char *);
 #define findVarInFrame3		Rf_findVarInFrame3
 #define GetArrayDimnames	Rf_GetArrayDimnames
 #define getAttrib		Rf_getAttrib
+#define getCharCE		Rf_getCharCE
 #define GetColNames		Rf_GetColNames
 #define GetMatrixDimnames	Rf_GetMatrixDimnames
-#define GetOption		Rf_GetOption
+#define GetOption1		Rf_GetOption1
 #define GetOptionDigits		Rf_GetOptionDigits
 #define GetOptionWidth		Rf_GetOptionWidth
+#define GetOption		Rf_GetOption
 #define GetRowNames		Rf_GetRowNames
 #define gsetVar			Rf_gsetVar
 #define inherits		Rf_inherits
 #define install			Rf_install
 #define isArray			Rf_isArray
+#define isBasicClass            Rf_isBasicClass
 #define isComplex		Rf_isComplex
 #define isEnvironment		Rf_isEnvironment
 #define isExpression		Rf_isExpression
@@ -873,6 +929,7 @@ int R_system(const char *);
 #define isNewList		Rf_isNewList
 #define isNull			Rf_isNull
 #define isNumeric		Rf_isNumeric
+#define isNumber		Rf_isNumber
 #define isObject		Rf_isObject
 #define isOrdered		Rf_isOrdered
 #define isPairList		Rf_isPairList
@@ -894,6 +951,8 @@ int R_system(const char *);
 #define lang2			Rf_lang2
 #define lang3			Rf_lang3
 #define lang4			Rf_lang4
+#define lang5			Rf_lang5
+#define lang6			Rf_lang6
 #define lastElt			Rf_lastElt
 #define lcons			Rf_lcons
 #define length(x)		Rf_length(x)
@@ -902,10 +961,15 @@ int R_system(const char *);
 #define list2			Rf_list2
 #define list3			Rf_list3
 #define list4			Rf_list4
+#define list5			Rf_list5
 #define listAppend		Rf_listAppend
 #define match			Rf_match
+#define matchE			Rf_matchE
 #define mkChar			Rf_mkChar
-#define mkCharEnc		Rf_mkCharEnc
+#define mkCharCE		Rf_mkCharCE
+#define mkCharLen		Rf_mkCharLen
+#define mkCharLenCE		Rf_mkCharLenCE
+#define mkNamed			Rf_mkNamed
 #define mkString		Rf_mkString
 #define namesgets		Rf_namesgets
 #define ncols			Rf_ncols
@@ -918,7 +982,9 @@ int R_system(const char *);
 #define psmatch			Rf_psmatch
 #define PrintValue		Rf_PrintValue
 #define protect			Rf_protect
+#define reEnc			Rf_reEnc
 #define rownamesgets		Rf_rownamesgets
+#define S3Class                 Rf_S3Class
 #define ScalarComplex		Rf_ScalarComplex
 #define ScalarInteger		Rf_ScalarInteger
 #define ScalarLogical		Rf_ScalarLogical
@@ -932,6 +998,8 @@ int R_system(const char *);
 #define StringBlank		Rf_StringBlank
 #define substitute		Rf_substitute
 #define translateChar		Rf_translateChar
+#define translateChar0		Rf_translateChar0
+#define translateCharUTF8      	Rf_translateCharUTF8
 #define type2char		Rf_type2char
 #define type2str		Rf_type2str
 #define unprotect		Rf_unprotect
@@ -964,6 +1032,7 @@ Rboolean Rf_isLanguage(SEXP);
 Rboolean Rf_isList(SEXP);
 Rboolean Rf_isMatrix(SEXP);
 Rboolean Rf_isNewList(SEXP);
+Rboolean Rf_isNumber(SEXP);
 Rboolean Rf_isNumeric(SEXP);
 Rboolean Rf_isPairList(SEXP);
 Rboolean Rf_isPrimitive(SEXP);
@@ -979,6 +1048,8 @@ SEXP	 Rf_lang1(SEXP);
 SEXP	 Rf_lang2(SEXP, SEXP);
 SEXP	 Rf_lang3(SEXP, SEXP, SEXP);
 SEXP	 Rf_lang4(SEXP, SEXP, SEXP, SEXP);
+SEXP	 Rf_lang5(SEXP, SEXP, SEXP, SEXP, SEXP);
+SEXP	 Rf_lang6(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 SEXP	 Rf_lastElt(SEXP);
 SEXP	 Rf_lcons(SEXP, SEXP);
 R_len_t  Rf_length(SEXP);
@@ -986,7 +1057,9 @@ SEXP	 Rf_list1(SEXP);
 SEXP	 Rf_list2(SEXP, SEXP);
 SEXP	 Rf_list3(SEXP, SEXP, SEXP);
 SEXP	 Rf_list4(SEXP, SEXP, SEXP, SEXP);
+SEXP	 Rf_list5(SEXP, SEXP, SEXP, SEXP, SEXP);
 SEXP	 Rf_listAppend(SEXP, SEXP);
+SEXP	 Rf_mkNamed(SEXPTYPE, const char **);
 SEXP	 Rf_mkString(const char *);
 int	 Rf_nlevels(SEXP);
 SEXP	 Rf_ScalarComplex(Rcomplex);
